﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;

namespace UnicodeSeqChecker.Class
{
    public class UnicodeSeqChecker
    {
        public string CheckSyntax(string content,bool autoCorrect)
        {
            string curChar = " ";
            string output = " ";

            int curCharCode;
            
            // Check if any previously checked data
            content = content.Replace("<", "");
            content = content.Replace(">", "");

            bool mismatchFound = false;
            bool outputDone = false;
          
            string nextSeq = " ";
            string prevSeq = " ";
            string nextnextSeq = " ";
            string prevprevSeq = " ";
            
            if (autoCorrect) 
               content = CorrectSequence(content);

            var arrChars = content.ToCharArray();

            
            for (int i = 0; i < arrChars.Length; i++)
            {
                outputDone = false;
                curChar = arrChars[i].ToString();
                curCharCode = (int)arrChars[i];



                // Rule : Tamil Ksha found with ZWNJ
                // Ksha formation is க் + <ZWNJ> + ஷ
                if (IsZWNJ(curCharCode))
                {
                    // Get the Next character to find its ஶ,ஷ
                    if (i + 1 < arrChars.Length)
                        nextSeq = content.Substring(i + 1, 1);
                    else
                        nextSeq = " ";

                    // Get the Previous two character to find whether its க் 
                    if (i - 2 >= 0)
                        prevSeq = content.Substring(i - 2, 2);
                    else
                        prevSeq = " ";

                    // If not (previous seq <> க் and next seq <> ஶ or ஷ) then its a garbage character
                    if (!(IsTamilPureConsonant_Ka(prevSeq) & IsTamilConsonant_sha(nextSeq)))
                        mismatchFound = true;
                    else
                    {
                        outputDone = true;
                        output = output + curChar + nextSeq;
                        i = i + 1;
                    }
                }

                // Rule : If Join Characters found with ZWJ
                // Conjunct Rule  : Sinhala Pure Consonant + <ZWJ> + Sinhala Consonant
                // Touching Rules : Sinhala Consonant + <ZWJ> + ් + Consonant 
                else if (IsZWJ(curCharCode))
                {
                    // Get the Next sequence to find whether its Sinhala Consonant
                    if (i + 1 < arrChars.Length)
                        nextSeq = arrChars[i + 1].ToString();
                    else
                        nextSeq = " ";

                    if (IsSinhalaConsonant(nextSeq))
                    {
                        // Get the Previous two character to find whether its Sinhala Pure Consonant 
                        if (i - 2 >= 0)
                            prevSeq = content.Substring(i - 2, 2);
                        else
                            prevSeq = " ";

                        if (!IsSinhalaPureConsonant(prevSeq))
                            mismatchFound = true;
                        else
                        {
                            outputDone = true;
                            output = output + curChar + nextSeq;
                            i = i + 1; // Since we know the next char is a consonant move to next char
                        }
                    }
                    else if (IsSinhalaAlKirima(nextSeq))
                    {
                        if (i - 1 >= 0)
                            prevSeq = content.Substring(i - 1, 1);
                        else
                            prevSeq = " ";

                        if (i + 2 < arrChars.Length) // Next Next char
                            nextnextSeq = content.Substring(i + 2, 1);
                        else
                            nextnextSeq = " ";

                        if (!(IsSinhalaConsonant(nextnextSeq) & IsSinhalaConsonant(prevSeq)))
                            mismatchFound = true;
                        else
                        {
                            outputDone = true;
                            output = output + curChar + nextSeq + nextnextSeq;
                            i = i + 2;
                        }
                    }
                    else
                        mismatchFound = true;
                }

                // Rule : SemiConsonant can be accompanied by a consonant or consonant + Vowel Modifier sequence 
                else if(IsSinhalaSemiConsonant(curChar))
                {
                    if (i - 1 >= 0)
                        prevSeq = content.Substring(i - 1, 1);
                    else
                        prevSeq = " ";
                    
                    if (i - 2 >= 0)
                        prevprevSeq = content.Substring(i - 2, 1);
                    else
                        prevprevSeq = " ";


                    if (!(IsSinhalaConsonant(prevSeq)))
                    {
                        if (!(IsSinhalaConsonant(prevprevSeq) & IsSinhalaVowelModifier(prevSeq)))
                            mismatchFound = true;
                    }      
                }

                // Rule : Is Sinhala VowelModifier followed by Sinhala Consonant
                else if (IsSinhalaVowelModifier(curChar))
                {
                    if (i - 1 >= 0)
                        prevSeq = content.Substring(i - 1, 1);
                    else
                        prevSeq = " ";

                    if (!IsSinhalaConsonant(prevSeq))
                        mismatchFound = true;
                }

                // Rule : Is Tamil VowelModifier followed by Tamil Consonant
                else if (IsTamilVowelModifier(curChar))
                {
                    if (i - 1 >= 0)
                        prevSeq = content.Substring(i + 1, 1);
                    else
                        prevSeq = " ";

                    if (!IsTamilConsonant(prevSeq))
                        mismatchFound = true;
                }

                else if (IsNonStandardCharacter(curChar))
                    mismatchFound = true;

                if (mismatchFound)
                {
                    if (!autoCorrect)
                        output = output + "<" + curChar + ">";
                    mismatchFound = false;
                }
                else if (!outputDone)
                    output = output + curChar;
            }
            return output.ToString().Trim();
        }

        private bool IsZWNJ(int charCode)
        {
            if (charCode == 8204)
                return true;
            else
                return false;
        }
        private bool IsZWJ(int charCode)
        {
            if (charCode == 8205)
                return true;
            else
                return false;
        }

        private bool IsSinhalaAlKirima(string curChar)
        {
            string AL_KIRIMA = "්";
            if (AL_KIRIMA.Equals(curChar))
                return true;
            else
                return false;
        }


        private bool IsTamilPureConsonant_Ka(string seq)
        {
            string TAMIL_PURE_CONSONANT_KA = "க்";
            if (TAMIL_PURE_CONSONANT_KA.Equals(seq))
                return true;
            else
                return false;
        }

        // Add a speacial character since 8205 nad other characters returns a 0 index always"
        private bool IsTamilConsonant_sha(string consonant)
        {
            string TAMIL_CONSONANT_SHA = "஖ஷ";
            if (TAMIL_CONSONANT_SHA.IndexOf(consonant) > 0)
                return true;
            else
                return false;
        }

        private bool IsSinhalaConsonant(string curChar)
        {
            string CONSONANTS = "஖කඛගඝඞඟචඡජඣඤඥඦටඨඩඪණඬතථදධනඳපඵබභමඹයරලවශෂසහළෆ";

            if (CONSONANTS.IndexOf(curChar) > 0)
                return true;
            else
                return false;
        }

        private bool IsSinhalaSemiConsonant(string curChar)
        {
            string SEMI_CONSONANTS = "஖ඃං";

            if (SEMI_CONSONANTS.IndexOf(curChar) > 0)
                return true;
            else
                return false;
        }

        private bool IsSinhalaPureConsonant(string seq)
        {
            string CONSONANTS = "஖කඛගඝඞඟචඡජඣඤඥඦටඨඩඪණඬතථදධනඳපඵබභමඹයරලවශෂසහළෆ";
            string ALKIRIMA = "්";

            if (seq.Length == 2)
            {
                string consonant = seq.Substring(0, 1);
                string alkirima = seq.Substring(1, 1);

                if ((CONSONANTS.IndexOf(consonant) > 0) & ALKIRIMA.Equals(alkirima))
                    return true;
                else
                    return false;
            }
            else
                return false;
        }

        private bool IsTamilConsonant(string curChar)
        {
            string CONSONANTS = "஖கஙசஜஞடணதநனபமயரறலளழவஶஷஸஹ";
            if (CONSONANTS.IndexOf(curChar) > 0)
                return true;
            else
                return false;
        }

        private bool IsTamilVowelModifier(string curChar)
        {
            string VOWEL_MODIFIER = "஖ாிீுூெேைொோௌ்ௗ";
            if (VOWEL_MODIFIER.IndexOf(curChar) > 0)
                return true;
            else
                return false;
        }
        private bool IsSinhalaVowelModifier(string curChar)
        {
            string VOWEL_MODIFIER = "஖්ාැෑිීුූෘෙේෛොෝෞෟෲෳ";
            if (VOWEL_MODIFIER.IndexOf(curChar) > 0)
                return true;
            else
                return false;
        }

        private bool IsNonStandardCharacter(string curChar)
        {
            string NON_STANDARD_CHARS = "஖ෳௗෟ"; ;
            if (NON_STANDARD_CHARS.IndexOf(curChar) > 0)
                return true;
            else
                return false;
        }

        private String CorrectSequence(string content)
        {
              content = content.Replace("අා", "ආ");
              content = content.Replace("අැ", "ඇ");
              content = content.Replace("අෑ", "ඈ");
              content = content.Replace("උෟ", "ඌ");
              content = content.Replace("එ්", "ඒ");
              content = content.Replace("ෙඑ", "ඓ");
              content = content.Replace("ඔ්", "ඕ");
              content = content.Replace("ඔෟ", "ඖ");
              content = content.Replace("ෙඑ", "ඓ");
              content = content.Replace("ේ", "ේ");
              content = content.Replace("ො", "ො");
              content = content.Replace("ෝ", "ෝ");
              content = content.Replace("ෝ", "ෝ");
              content = content.Replace("ෞ", "ෞ");
              content = content.Replace("ொ", "ொ");
              content = content.Replace("ோ", " ோ");
              content = content.Replace("ௌ", "ௌ");
              return content;
        }
    }
}