﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Runtime.InteropServices;
using System.Text;
using System.Web;
using System.Web.Mvc;
using Microsoft.Office.Interop.Excel;
using Novacode;

using ClosedXML.Excel;

namespace UnicodeSeqChecker.Controllers
{
    public class HomeController : BaseController
    {
        public ActionResult Index()
        {
            ViewBag.Message = "";
            return View();
        }
        public ActionResult Tamil()
        {
            ViewBag.Message = "";
            return View();
        }
        public ActionResult Sinhala()
        {
            ViewBag.Message = "";
            return View();
        }
      
        public ActionResult ChangeCurrentCulture(int id)
        {
            
            SessionManager.CurrentCulture = id;
            //
            // Cache the new current culture into the user HTTP session. 
            //
            Session["CurrentCulture"] = id;
            //
            // Redirect to the same page from where the request was made! 
            //

            if (id == 2)
            {
                return RedirectToAction("Sinhala");
            }
            else if (id == 1)
            {
                return RedirectToAction("Tamil");
            }
            else
            {
                return RedirectToAction("Index");
            }
        }

        public ActionResult Help()
        {
            ViewBag.Message = "";
            return View();
        }
        public ActionResult HelpTamil()
        {
            return View();
        }
        public ActionResult HelpSinhala()
        {
            return View();
        }

        public ActionResult OnlineUnicodeSeqChecker()
        {
            return View();
        }


        public ActionResult DocumentUnicodeSeqChecker(string id)
        {
            ViewBag.DownloadUrl = null;
            if (Session[id] != null)
            {
                ViewBag.FileName = Session[id].ToString();
            }
            return View();
        }

        public ActionResult Api()
        {
            ViewBag.Message = "";
            return View();
        }

        public ActionResult ApiTamil()
        {
            ViewBag.Message = "";
            return View();
        }
        public ActionResult ApiSinhala()
        {
            ViewBag.Message = "";
            return View();
        }
             
        public void CheckWordDocument(string path,bool autoCorrect)
        {
            DocX doc = DocX.Load(path);
            UnicodeSeqChecker.Class.UnicodeSeqChecker syntaxChecker = new UnicodeSeqChecker.Class.UnicodeSeqChecker();
               
            for (int i=0; i < doc.Paragraphs.Count ; i++)
            {
                if (!doc.Paragraphs[i].Text.Trim().Equals(""))
                { 
                   string convertedText = syntaxChecker.CheckSyntax(doc.Paragraphs[i].Text, autoCorrect);
                   doc.ReplaceText(doc.Paragraphs[i].Text, convertedText);
                   doc.Save();
                }
            }           
       }

       
        public void CheckExcelDocument(string path,bool autoCorrect)
        {
            var workbook = new XLWorkbook(path);

            for (int i=1;i <= workbook.Worksheets.Count; i++) 
            {
                var objWorksheet = workbook.Worksheet(i);

                var objFullRange = objWorksheet.RangeUsed();
                var objUsedRange = objWorksheet.Range(1, 1, objFullRange.RangeAddress.LastAddress.RowNumber,
                                                        objFullRange.RangeAddress.LastAddress.ColumnNumber);
                var iNumberOfcolumnsInTheExcel = objUsedRange.ColumnCount();
                UnicodeSeqChecker.Class.UnicodeSeqChecker syntaxChecker = new UnicodeSeqChecker.Class.UnicodeSeqChecker();
  
                for (int j = 1; j <= objFullRange.RangeAddress.LastAddress.RowNumber; j++)
                {
                    for (int k = 1; k <= objFullRange.RangeAddress.LastAddress.ColumnNumber; k++)
                    {
                        if (objWorksheet.Cell(j, k).Value != null)
                        {
                            string value = (string)objWorksheet.Cell(j, k).Value;
                            objWorksheet.Cell(j, k).Value = syntaxChecker.CheckSyntax(value, autoCorrect);
                        }
                    }
                }
            }

            workbook.Save();
            workbook = null;
        }
       
        public ActionResult Upload(HttpPostedFileBase file)
        {
            var downloadURL = string.Empty;

            bool autoCorrect = false ;
            if (Request.Form["auto"] != null)
                autoCorrect = true;

            var ticks = string.Empty;
            try
            {
                if (file != null && file.ContentLength > 0)
                {
                    ticks = DateTime.Now.Ticks.ToString();
                    var fileName = "SC_" + ticks + Path.GetExtension(file.FileName);
                    downloadURL = fileName;

                    Session.Add(ticks, downloadURL);

                    var path = Path.Combine(Server.MapPath("~/UploadedFiles"), fileName);

                    file.SaveAs(path);
                    string ext = Path.GetExtension(fileName);

                    var text = string.Empty;
                    if (ext == ".doc" || ext == ".docx")
                        CheckWordDocument(path,autoCorrect);
                    else if (ext == ".xls" || ext == ".xlsx")
                        CheckExcelDocument(path, autoCorrect);
                }   
            }
            catch (Exception e)
            {
                throw new Exception(e.Message + e.StackTrace);
            }

            var url = "/Home/DocumentUnicodeSeqChecker?id=" + ticks;
            return Redirect(url);
        }

        
        public FileResult Download(string fileName)
        {
            var path = Path.Combine(Server.MapPath("~/UploadedFiles"), fileName);

            string ext = Path.GetExtension(fileName);

            var text = string.Empty;
            string contentType = string.Empty ;
            if (ext == ".doc")
                contentType = "application/vnd.openxmlformats-officedocument.wordprocessingml.document";
            else if (ext == ".docx")
                contentType = "application/vnd.openxmlformats-officedocument.wordprocessingml.document";
            else if (ext == ".xlsx")
                contentType = "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet";
            else if (ext == ".xls")
                contentType = "application/vnd.ms-excel";
           
            var file = System.IO.File.Open(path, FileMode.Open, FileAccess.Read, FileShare.ReadWrite);
            var stream = (Stream)(file);
            return File(stream, contentType, fileName);
        }
    }
}
